part of 'vehicles_on_the_property.dart';

/// [VehicleHistoryScreen] is a StatefulWidget that displays a list of vehicles.

// ignore: must_be_immutable
class VehicleHistoryScreen extends StatefulWidget {

  VehicleHistoryScreen(this.model, {super.key});
  late VehicleModel model;
  final scaffoldKey = GlobalKey<ScaffoldState>();
  final builderKey = GlobalKey();

  @override
  State<VehicleHistoryScreen> createState() => _VehicleHistoryScreenState();

}

class _VehicleHistoryScreenState extends State<VehicleHistoryScreen>
    with Remotable {


  @override
  void dispose() {
    super.dispose();
    _scrollController.dispose();
  }

  @override
  void initState() {
    super.initState();
    _initializeScrollController();
  }

  @override
  Widget build(BuildContext context) {
    late final limitedHeaderTextSize =
        LimitedFontSizeUtil.getHeaderFontSize(context);
    late final double limitedBodyTextSize =
        LimitedFontSizeUtil.getBodyFontSize(context);

    return Scaffold(
      key: widget.scaffoldKey,
      body: Column(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          if (_hasData == false &&
              _pageNumber <= 1 &&
              _loading == false)
            _buildNoDataFound(context, limitedHeaderTextSize)
          else if (_hasData == true || _pageNumber >= 1)
            _buildHistoryList(context, limitedBodyTextSize),
          if (_hasData == true && _loading == true)
            _buildLoadingIndicator(context),
        ].addToStart(const SizedBox(height: 0)),
      ),
    );
  }

  Widget _buildNoDataFound(BuildContext context, double limitedHeaderTextSize) {
    return Expanded(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        mainAxisSize: MainAxisSize.max,
        children: [
          Center(
            child: Text(
              FFLocalizations.of(context).getVariableText(
                ptText: "Nenhum veículo encontrado!",
                enText: "No vehicle found",
              ),
              style: TextStyle(
                fontFamily: 'Nunito',
                fontSize: limitedHeaderTextSize,
              ),
            ),
          )
        ],
      ),
    );
  }

  Widget _buildHistoryList(BuildContext context, double limitedBodyTextSize) {
    return Expanded(
      child: FutureBuilder<void>(
        future: _fetch(),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.waiting) {
            return Center(child: CircularProgressIndicator());
          } else if (snapshot.hasError) {
            return Center(child: Text('Error: ${snapshot.error}'));
          } else {
            return ListView.builder(
              shrinkWrap: true,
              addAutomaticKeepAlives: true,
              restorationId: '',
              physics: const BouncingScrollPhysics(),
              controller: _scrollController,
              itemCount: _wrap.length,
              itemBuilder: (context, index)  =>  _generateItems(context, _wrap[index], index),
         
            );
          }
        },
      ),
    );
  }

  Widget _buildLoadingIndicator(BuildContext context) {
    return Container(
      padding: const EdgeInsets.only(top: 15, bottom: 15),
      child: Center(
        child: CircularProgressIndicator(
          valueColor: AlwaysStoppedAnimation<Color>(
            FlutterFlowTheme.of(context).primary,
          ),
        ),
      ),
    );
  }

  

}


mixin Remotable on State<VehicleHistoryScreen> {

  late ScrollController _scrollController;
  double offset = 0.0;
  List<dynamic> _wrap = [];
  int _pageNumber = 1;
  bool _hasData = true;
  bool _loading = true;
  int count = 0;

  Future<void> _fetch() async {
    if (!_hasData || !_loading) return;
    print('hasHasData');
    setState(() => _loading = true);
    try {

      var response = await PhpGroup.getVehiclesByProperty.call(_pageNumber.toString());
      final List<dynamic> vehicles = response.jsonBody['vehicles'] as List<dynamic>? ?? [];
      safeSetState(() => count = response.jsonBody['total_rows'] ?? 0);
      if (vehicles.isNotEmpty) {
        setState(() {
          _hasData = true;
          _loading = false;
          _wrap.addAll(vehicles);
        });
        
        return;
      }

      _showNoMoreDataSnackBar(context);

      setState(() {
        _hasData = false;
        _loading = false;
      });

      print('hasEmpty: ${_wrap.length}');
      return;
    } catch (e, s) {
      DialogUtil.errorDefault(context);
      LogUtil.requestAPIFailed("proccessRequest.php", "", "Consulta de Veículo", e, s);
      setState(() {
        _hasData = false;
        _loading = false;
      });
      print('hasError');
      return;
    }
  }

  void _showNoMoreDataSnackBar(BuildContext context) {
    final message = FFLocalizations.of(context).getVariableText(
      ptText: "Não há mais dados.",
      enText: "No more data.",
    );

    showSnackbar(context, message, true);
  }

  Widget _buildHeader(BuildContext context) {
    double limitedBodyTextSize = LimitedFontSizeUtil.getBodyFontSize(context);
    log('amountRegister: ${widget.model.amountRegister}');
    return Padding(
      padding: const EdgeInsets.only(right: 30, top: 10),
      child: Text(
        (widget.model.amountRegister == '0' ||
                widget.model.amountRegister == null)
            ? ''
            : "${FFLocalizations.of(context).getVariableText(ptText: "Quantidade de Veículos: ", enText: "Amount of Vehicles: ")}${widget.model.amountRegister}/$count",
        textAlign: TextAlign.right,
        style: TextStyle(
          fontFamily: 'Nunito',
          fontSize: limitedBodyTextSize,
        ),
      ),
    );
  }

 Widget?  _generateItems(BuildContext context, dynamic snapshot, int index)  {
    if (index == 0) return _buildHeader(context);
                
      final bool? isOwner = snapshot['isOwnerVehicle'];
      final IconData? iconData = isOwner == true ? Symbols.garage : Symbols.directions_car;
      final FreCardIcon? cardIcon = isOwner != null
          ? FreCardIcon(height: 50, width: 100, icon: Icon(iconData, size: 80, opticalSize: 10))
          : null;

      final String? tag = snapshot['tag'];
      final bool containTag = tag.isNotNullAndEmpty;
      final Map<String, String> labelsHashMap = _generateLabelsHashMap(context, snapshot, tag, containTag);

      final List<Map<String, Color>> statusHashMapList =   _generateStatusHashMapList(snapshot);

      Future<void> onTapCardItemAction() async {
         _handleCardItemTap(context, cardIcon, snapshot);
      }

      final statusLinkedHashMap = statusHashMapList.map((map) => LinkedHashMap<String, Color>.from(map)).toList();
      final length = statusLinkedHashMap.expand((e) => [e.length]);

       return CardItemTemplateComponentWidget(
        icon: cardIcon,
        labelsHashMap: labelsHashMap,
        statusHashMap: statusHashMapList,
        onTapCardItemAction: onTapCardItemAction,
        itemWidthFactor: length == 1 ? 0.25 : 0.50,
      );
    }
  

  Map<String, String> _generateLabelsHashMap(BuildContext context,
      Map<String, dynamic> uItem, String? tag, bool containTag) {
    return {
      '${FFLocalizations.of(context).getVariableText(ptText: "Placa", enText: "License Plate")}:':
          uItem['licensePlate'] ?? '',
      '${FFLocalizations.of(context).getVariableText(ptText: "Modelo", enText: "Model")}:':
          uItem['model'] ?? '',
      '${FFLocalizations.of(context).getVariableText(ptText: "Proprietário", enText: "Owner")}:':
          uItem['personName'] ?? '',
      if (containTag)
        '${FFLocalizations.of(context).getVariableText(ptText: "Tag", enText: "Tag")}:':
            tag ?? '',
    };
  }

 List<Map<String, Color>> _generateStatusHashMapList(
      Map<String, dynamic> uItem)  {
    final statusHashMap =
         widget.model.generateStatusColorMap(uItem, false);
    return statusHashMap != null ? [statusHashMap] : [];
  }

  Future<void> _handleCardItemTap(BuildContext context, FreCardIcon? cardIcon,
      Map<String, dynamic> uItem) async {
    try {
      final dialogContent =  widget.model.buildVehicleDetails(
        icon: cardIcon,
        item: uItem,
        context: context,
        model: widget.model,
      );

      await showDialog(
        useSafeArea: true,
        context: context,
        builder: (context) =>
            Dialog(alignment: Alignment.center, child: dialogContent),
      ).whenComplete(() {
        safeSetState(() {
          _pageNumber = 1;
          _fetch();
        });
      });
    } catch (e, s) {
      DialogUtil.errorDefault(context);
      LogUtil.requestAPIFailed(
          "proccessRequest.php", "", "Consulta de Veículos", e, s);
      safeSetState(() {
        _hasData = false;
        _loading = false;
      });
    }
  }

  void _initializeScrollController() {
    _scrollController = ScrollController(keepScrollOffset: true, initialScrollOffset:  offset)
      ..addListener(() {
        print('ScrollController');
        if(!_hasData) return;
        if 
        // (_scrollController.position.pixels >= _scrollController.position.maxScrollExtent) 
        (_scrollController.position.atEdge && _scrollController.position.pixels != 0)
        {
          print('ScrollController -> loadMore');
          offset = _scrollController.offset;
          _loadMore();
        }
      },);
  }

  void _loadMore()  {
    if (_hasData) {
      _pageNumber+=1;
      _loading = true;
      _fetch();
    }
  }

}
